#ifndef _GO_TO_BASE_OPTION_H_
#define _GO_TO_BASE_OPTION_H_

#include "PassiveOption.h"

const int GO_TO_BASE_OPTION_ID = 704;


/**
  Diese Klasse steht f\"ur den Wunsch auf eine Basisposition zur\"uckzulaufen.
  Diese Option bekommt eine hohe Aktivierung, wenn es keinen Sinn f\"ur den
  Spieler hat direkt oder indirekt ins Spiel einzugreifen. Dies ist meist der
  Fall, wenn der Ball sehr weit vom Spieler entfernt ist. Bei Auswahl dieses
  Wunsches l\"auft der Spieler zu einer Basisposition zur\"uck.

  In besonderen Spielsituationen (Anspiel, etc.) werden andere Basispositionen
  angenommen.

  In den Spielzust\"anden in dennen das beamen m\"oglich ist, wird zur
  Basisposition gebeamt.

  {\small {\bf Quelltext}

  [\URL[Header]{src/GoToBaseOption.h} | [\URL[Implementierung]{src/GoToBaseOption.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: GoToBaseOption.h,v $
  *   $Revision: 1.10 $
  *   $Date: 1998/08/14 13:20:37 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class GoToBaseOption : public PassiveOption
{
public:
  /** Erzeugt eine GoToBaseOption. */
  GoToBaseOption();
  
  /** Erzeugt einen Plan zur Umsetzung der Absicht.  */
  Plan generatePlan() const;

  /** Prueft, ob die Absicht tatsaechlich umsetzbar ist. 
    Dies ist nicht der Fall, wenn wir schon an der tatsaechlich angestrebten
    Position sind. Normalerweise ist das die basePosition, es sei denn, wir
    haben das AvoidOffsideConstraint akzeptiert.
    @return YES oder NO */
  virtual bool isPossible() const;  
  
  /** Beschreibung als String f\"ur die Ausgabe auf einem Terminal. */
  virtual String toString() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
    darauf zur\"uck. Diese Methode muss fuer JEDE Unterklasse neu definiert 
    werden. */
  virtual StorableObject* copy() const;
  
private:
  
  /** Der Grundnutzen des Wunsches. Im normalen Spiel ist der Nutzen umso
    gr\"osser, je weiter der Spieler von seiner Position entfernt ist.
    In besonderen Spielsituation kann der Nutzen wesentlich h\"oher sein.
    @return Ein Wert zwischen MAX_UTILITY und NEUTRAL_UTILITY. */
  int baseUtility() const;

  /** Setzt die HomePosition beim Freisto\"s ec. des Gegners so, da\"s der
    Verteidiger besser zur Freisto\3position steht. */
  void resetDefenderTarget() const;

  /** Setzt die Variable die aussagt, ob das Freischu\"sziel bestimmt wurde. */
  void setIsFreeKickTargetSelected( bool isFreeKickTargetSelected ) const;

  /** Erfragt die Variable die aussagt, ob das Freischu\"sziel bestimmt
    wurde. */
  bool isFreeKickTargetSelected() const;

  /** Setzt die HomePosition um, wenn der Ball weit entfernt in der
      gegenerischen Haelfte liegt. */
  void resetTargetIfMoveForwardSituation() const;

  /** Setzt die Variable die aussagt, ob das Vorlaufziel bestimmt wurde. */
  void setIsMoveForwardTargetSelected( bool isMoveForwardTargetSelected) const;

  /** Erfragt die Variable die aussagt, ob das Vorlaufziel bestimmt wurde. */
  bool isMoveForwardTargetSelected() const;

  /** Variable die aussagt, ob ein Freischu\"sziel bestimmt wurde. */
  mutable bool theIsFreeKickTargetSelected;

  /** Variable die aussagt, ob das Vorlaufziel bestimmt wurde. */
  mutable bool theIsMoveForwardTargetSelected;
};

inline void GoToBaseOption::setIsFreeKickTargetSelected( 
    bool isFreeKickTargetSelected ) const {
  theIsFreeKickTargetSelected = isFreeKickTargetSelected;
}

inline bool GoToBaseOption::isFreeKickTargetSelected() const {
  return theIsFreeKickTargetSelected;
}

inline void GoToBaseOption::setIsMoveForwardTargetSelected( 
    bool isMoveForwardTargetSelected) const {
  theIsMoveForwardTargetSelected = isMoveForwardTargetSelected;
}

inline bool GoToBaseOption::isMoveForwardTargetSelected() const {
  return theIsMoveForwardTargetSelected;
}

#endif
