#ifndef _LOG_H_
#define _LOG_H_

#include <iostream.h>
#include "Array.h"

#define ConfigurationTest
//#define SensorTest
 
class String;
class Configuration;

/**
  Verwaltet die Zeichenketten, die der Agent ausgeben m\"ochte.
  Abh\"angig vom eingestellten DebugModus und dazu geh\"origen  DebugLevel
  werden Zeilen ausgegeben oder nicht. \\
  Der DebugModus setzt sich aus mehreren Modi zusammen, deren DebugLevel
  im Konfigurationsfile des Agenten festgelegt werden.
  Warnungen und Fehler (Debug-Informationen der Level 0 und 1) werden immer
  ausgegeben.

  Modi:
  \begin{tabular}{ll}
  Communicator & Debug-Informationen des Communicators \\
  Sensor & Debug-Informationen der Sensoren \\
  ... & ...
  \end{tabular}

  DebugLevel:
  \begin{tabular}{ll}
  0     & Fehler ausgeben \\
  1     & zus\"atzlich Warnungen ausgeben \\
  2-4   & zus\"atzlich Funktionseintritte und Funktionsbeendigungen \\
  5-7   & zus\"atzlich Ergebnisse von Funktionen (wichtige Informationen,
            wie see, hear, utility-Werte) \\
  8-10  & zus\"atzlich Informationen zu Zwischenschritten \\
  11-15 & zus\"atzlich weitere Informationen
  \end{tabular} \\
  Wichtige Funktionseintritte sollten beispielsweise mit einem 
  Level von 2, unwichtige Funktionseintritte dagegen mit einen Level von 4
  aktiviert werden. \\

  Beispiel zur Benutzung:
  \begin{verbatim}
  INFO( Log::WORLD_MODEL, 7, "Hallo" )
  \end{verbatim}
  In diesem Fall wird "Hallo" ausgegeben, wenn der DebugLevel f\"ur das
  Weltmodell auf 7 oder gr\"o\3er gesetzt wurde.

  Wenn man viele Debug-Ausschriften ausgeben m\"ochte, dann bietet sich 
  diese Benutzung an:
  \begin{verbatim}
  if ( IS_ACTIVE( Log::WORLD_MODEL, 7 ) ) {
    globalLog->info( "Hallo" );
    globalLog->info( "nochmal Hallo" );
  }
  \end{verbatim}

  Die Markos schreiben immer ins globalLog.

  {\small {\bf Quelltext}

  [\URL[Header]{src/Log.h} | [\URL[Implementierung]{src/Log.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Log.h,v $
  *   $Revision: 1.27 $
  *   $Date: 1998/08/14 13:20:47 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class Log
{
public:

  /** @name M\"ogliche Modi f\"urs Logging. */
  //@{
    /// beliebiger Modus
    const int ANY = 0;
    /// Modus f\"ur den Communicator
    const int COMMUNICATOR = 1;
    /// Modus f\"ur die Configuration
    const int CONFIGURATION = 2;
    /// Modus f\"ur die Sensoren
    const int SENSORS = 3;
    /// Modus f\"ur die Effektoren
    const int EFFECTORS = 4;
    /// Modus f\"ur die Situation
    const int SITUATION = 5;
    /// Modus f\"ur das Weltmodel
    const int WORLD_MODEL = 6;
    /// Modus f\"ur den Timer
    const int TIMER = 7;
    /// Modus f\"ur den Deliberator
    const int DELIBERATOR = 8;
    /// Modus f\"ur die Constraints
    const int CONSTRAINTS = 9;
    /// Modus f\"ur die PassiveOptions
    const int PASSIVE_OPTIONS = 10;
    /// Modus f\"ur die ActiveOptions
    const int ACTIVE_OPTIONS = 11;
    /// Modus f\"ur alle ModelledObjects
    const int MODELLED_OBJECTS = 12;
    /// Modus f\"ur die Skills
    const int SKILLS = 13;
    /// Anzahl aller Modi
    const int MODE_COUNT = 14;
  //@}

  /** @name Log anlegen und initialisieren */
  //@{
    /** Konstruktor. Setzt das DebugLevel und den Ausgabe-Stream. Weiterhin
      aktiviert er alle Modi f\"ur die Ausgabe mit dem angegebenen
      DebgugLevel. */
    Log( int debugLevel = 0, ostream& target = cout );
  
    /** Initialisiert das Log-Objekt neu. Initialisiert, mit dem als Parameter
      angegebenen Configurationsobjekt, die Debug-Level f\"ur alle Ausgabemodi.
      @param aConfigurationObject Pointer auf das Configuration-Objekt. */
    void init( Configuration *aConfigurationObject );
  //@}

  /** @name Anfragen ob Modus mit DebugLevel aktiv */
  //@{
    /** Fragt ob der als Paramter angegebene Modus mit dem angebenen 
      DebugLevel aktiv ist. Der Modus ist aktiv, wenn sein eingestellter
      DebugLevel gr\"o\3er oder gleich dem angegebenen DebugLevel ist.
      Diese Methode sollte in einer if-Anweisung, die \Ref{info} Anweisungen
      kapseln.
      @param mode Modus f\"ur den die Aktivit\"at erfragt wird
      @param debugLevel Debug-Level f\"ur den die Aktivit\"at erfragt wird */
    bool isActive( int mode, int debugLevel );
  //@}

  /** @name Ins Log schreiben. */
  //@{
    /** Meldet Fehler. Diese Methode sollte nie von \Ref{isActive} gekapselt
      werden. */
    void error( const String& message );

    /** Meldet Warnung. */
    void warning( const String& message );  
  
    /** Debug-Info. Diese Metode sollte immer von \Ref{active} gekapselt
      werden.
     @param message Die Nachricht die ausgegeben werden soll.  */
    void info( const String& message );
  //@}

  /** @name Log-Makros */
  //@{
    #if DEBUG_ON==1
    
    /** Fragt, ob ein Debug-Mode aktiviert ist.
      Wenn ja, wird die der folgende Block ausgefuehrt, andernfalls nicht. 
      @param debugMode Modus f\"ur den die Aktivit\"at erfragt wird
      @param debugLevel Debug-Level f\"ur den die Aktivit\"at erfragt wird */
    #define IS_ACTIVE( debugMode, debugLevel ) \
      ( globalLog->isActive( (debugMode), (debugLevel) ) )
    
    /** Meldet Fehler. Eine Ausgabe erfolgt immer im \Ref{globalLog}. */
    #define ERROR( message ) \
    { \
      globalLog->error( message ); \
    }

    /** Meldet Warnung. Eine Ausgabe erfolgt im \Ref{globalLog}, wenn der
      angegebene Modus debugMode
      im Log-Objekt mit einen debugLevel von 1 oder h\"oher eingestellt ist.
      @param debugMode Der Modus bei dem ausgegeben werden soll.
      @param message Die Nachricht die ausgegeben werden soll. */
    #define WARNING( debugMode, message ) \
    { \
      if( globalLog->isActive( debugMode, 1 ) ) { \
        globalLog->warning( message ); \
      } \
    }

    /** Debug-Info. Eine Ausgabe erfolgt im \Ref{globalLog}, wenn der
      angegebene Modus debugMode
      im Log-Objekt mit den angegebenen debugLevel aktiv ist.
      Der Modus ist aktiv, wenn sein eingestellter
      DebugLevel gr\"o\3er oder gleich dem angegebenen DebugLevel ist.
      @param debugMode Der Modus bei dem ausgegeben werden soll.
      @param debugLevel Der Debug-Level ab dem ausgeben werden soll.
      @param message Die Nachricht die ausgegeben werden soll. */
    #define INFO( debugMode, debugLevel, message ) \
    { \
      if( globalLog->isActive( debugMode, debugLevel ) ) { \
        globalLog->info( message ); \
      } \
    }
    #else
    #define ERROR( message )
    #define WARNING( debugMode, message )
    #define INFO( debugMode, debugLevel, message )
    #define IS_ACTIVE( debugMode, debugLevel ) (NO) 
    #endif

  //@}

  /** Beschreibung als String fuer die Ausgabe. */
  String toString() const;

private:
  /** Array, welches f\"ur jeden Modus enth\"alt, wie stark er aktiviert
    wurde. */
  Array<int> theModeActivation;

  /** Der Stream auf dem die Ausgabe erfolgt. */
  ostream *theTargetStream;
};

extern Log *globalLog; 

#endif


