#ifndef _EXCEPTION_H_
#define _EXCEPTION_H_

class String;

#include <stream.h>
#include "Common.h"

/**
  Laufzeitfehlerklasse. Besitzt einen String, der den Fehler beschreibt.
  Wird die Exception nicht mit dem C++-Schl\"usselwort throw, sondern mit
  \Ref{throwException}() geworfen, kann sie au\3erdem den Quelltext-Dateinamen
  und die Zeile, in der sie geworfen wurde, liefern. Hierzu kann entweder
  \Ref{toString}() oder direkt der Umleitungsoperator << verwendet werden.

  Damit Laufzeitfehler auch in der String-Klasse vorkommen k\"onnen, mu\3 der
  Fehlerstring als Pointer implementiert sein.
  
  Die Klasse ist zur Vererbung geeignet.

  {\small {\bf Quelltext}

  [\URL[Header]{src/Exception.h} | [\URL[Implementierung]{src/Exception.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Exception.h,v $
  *   $Revision: 1.8 $
  *   $Date: 1998/08/14 13:20:29 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class Exception
{
public:
  /** @name Exceptions erzeugen und zerst\"oren */
  //@{
    /** Erzeugen einer neuen Exception ohne Beschreibung. */
    Exception();
    
    /** Erzeugen einer neuen Exception mit einer Beschreibung. 
      Die Beschreibung sollte weder Zeilennummer oder Dateiname enthalten, da
      diese Daten automatisch beim Aufruf von \Ref{throwException} "erhoben"
      werden.
      @param aDescription Eine Beschreibung des Zustandes, der das Programm zum 
        Werfen der Exception veranla\3t hat.*/
    Exception( const String& aDescription );

    /** Destruktor */
    virtual ~Exception();
  //@}
    
  
  /** @name Informationen setzen */  
  //@{
    /** Setzt den Qelltext-Dateinamen, in dem die Exception erzeugt wurde. 
      Wird von \Ref{throwException} aufgerufen. 
      @param fileName Quellcode-Dateiname, der per Konvention bis auf die
        Endung mit dem Klassennamen \"ubereinstimmt. */
    static void setFileName( const String& fileName );
  
    /** Setzt die Beschreibung des Fehlers., in dem die Exception erzeugt wurde. 
      Die Beschreibung sollte weder Zeilennummer oder Dateiname enthalten, da
      diese Daten automatisch beim Aufruf von \Ref{throwException} "erhoben"
      werden. 
      @param aDescription Eine Beschreibung des Zustandes, der das Programm zum 
        Werfen der Exception veranla\3t hat.*/
    void setDescription( const String& description );
  
    /** Setzt die Zeilennummer, in der die Exception erzeugt wurde.
      Wird von \Ref{throwException} aufgerufen. 
      @param Die Zeilennummer in der Quelldatei */
    static void setLineNumber( int lineNumber );
  //@}

  /** @name Informationen abfragen */
  //@{
    /** Liefert eine m\"oglichst ausf\"uhrliche Beschreibung der Fehlermeldung.
      Je nach vorliegenden Informationen wird eine mehr oder weniger
      ausf\"uhrliche Meldung erzeugt. Diese Methode ist NICHT virtual.
      Sie gibt nur das f\"ur alle Exceptions gleiche Ausgabeformat aus.*/
    const String toString() const;

    /** Liefert den Fehlertext der Exception 
      Wenn der Text leer ist, wird 'no description' geliefert.
      @return String mit der Fehlerbeschreibung. */
    const String description() const;
        
    /** Gibt die Exception auf einem Stream aus. 
      Normalerweise wird diese Methode nicht direkt, sondern nur \"uber << 
      aufgerufen. */
    void printOn (ostream& = cout) const;
  //@}
    
private:
  /** Zeiger auf String mit Fehlermeldung.  */
  String *theDescription;
  /** Zeiger auf String mit Dateiname, wo die Exception geworfen wurde. */
  static String *theFileName;
  /** Zeilennummer, in der die Exception geworfen wurde.*/
  static int theLineNumber;
  
  static void resetFileName();
};

/** Wirft eine Exception. 
  Setzt au\3erdem Dateinamen und Zeilennummer des Werfens. Daher sollte immer 
  dieser Makro und nicht das Schl\"usselwort throw verwendet werden. 
    @param anException Exception-Objekt, das geworfen wird. */
#define throwException(anException) \
{ \
  Exception::setFileName( __FILE__ ); \
  Exception::setLineNumber( __LINE__ ); \
  throw anException; \
}


inline ostream& operator<< (ostream& aStream, const Exception& object){
  object.printOn( aStream );
  return aStream;
}


#endif


