#ifndef _GOALIE_OPTION_H_
#define _GOALIE_OPTION_H_

#include "PassiveOption.h"
#include "StraightLine.h"

const int GOALIE_OPTION_ID = 721;


/**
  Diese Klasse ist fuer das Torwartverhalten zustaendig. 
  
  Zunaechst wird
  immer der Ball beobachtet. Beim Eintritt in die eigene Haelfte wird die
  Moeglichkeit eines Torschusses geprueft. Besteht die Gefahr, da\3 der
  Gegner als naechstes den Ball bekommt, wird ein idealer Punkt innerhalb
  eines 7 m Radius' um den Tormittelpunkt  berechnet und dorthin gegangen.
  Unterschreitet der Gegner 25m Distanz und kann kein eigener Spieler mit
  grosser Wahrscheinlichkeit noch zwischen Tor und Gegner kommen, l"auft der
  Torwart bis 2m vor die Strafraumgrenze dem Ball entgegen. Kann doch noch ein
  eigener Verteidiger dazwischen kommen, dann wird nur 2m entgegengelaufen.
  Wird der Ball im Strafraum sein, versucht der Torwart ihn mit den F"ussen
  zu erreichen, geht das nicht, versucht er ihn zu catchen.
  Unterschreitet ein Gegner die Distanz von 19 m (entspricht einem Ballweg
  von 9 Takten bei einem Schuss mit 2.7) wird der Blickwinkel auf 45
  verringet.

  {\small {\bf Quelltext}

  [\URL[Header]{src/GoalieOption.h} | [\URL[Implementierung]{src/GoalieOption.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: GoalieOption.h,v $
  *   $Revision: 1.8 $
  *   $Date: 1998/08/14 13:20:41 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class GoalieOption : public PassiveOption
{
public:
  /** Erzeugt eine GoalieOption. */
  GoalieOption();
  
  /** Erzeugt einen Plan zur Umsetzung der Absicht.  */
  Plan generatePlan() const;

  /** Beschreibung als String fuer die Ausgabe auf einem Terminal. */
  virtual String toString() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
      darauf zurueck. Diese Methode muss fuer JEDE Unterklasse neu definiert 
      werden. */
  virtual StorableObject* copy() const;

private:
  /** Distanz vom Tor, in der sich der Torwart normalerweise aufhaelt */
  double goalieDistFromGoal;
  /** Bestimmt ab welcher Entfernung eines Mitspielers der Torwart
      kein Intercept macht. */
  int isBetterAsGoalie;
  /** Flag f"ur Goalie-Intercept */
  mutable bool theGoalieIntercept;
  /** Flag f"ur GoToGolieHomePosition */
  mutable bool theGoToGoalieHomePosition;
  /** Flag f"ur sofortiges catch */
  mutable bool theCatchAtOnce;
  /** Flag f"ur Positionsspiel */
  mutable bool thePositionPlay;
  /** Flag f"ur 90 Gradwinkel Sichtwinkel */
  mutable bool theNormalView;
  /** simStep des letzten Freistoss */
  static int theFreeKickSimStep;
  
  
  /** Setzen des doIntercept-Flags */
  void setGoalieIntercept( bool on ) const;
  /** Setzen des toGolieHomePosition-Flags*/
  void setGoToGoalieHomePosition( bool on ) const;
  /** Setzen des catchAtOnce-Flags */
  void setCatchAtOnce( bool on ) const;
  /** Setzen des positionPlay-Flags */
  void setPositionPlay( bool on ) const;
  /** Setzen des Normal-View-Flags */
  void setNormalView( bool on ) const;
  /** Setzen des Zeitpunktes des letzten Freistoss */
  void setFreeKickSimStep( int newSimStep ) const;
  
  /** Werte abfragen */
  /** Flags abfragen */
  bool isCatchAtOnce() const;
  bool isGoToGoalieHomePosition() const;
  bool isGoalieIntercept() const;
  bool isPositionPlay() const;
  bool isNormalView() const;
  int freeKickSimStep() const;

  /** Ist eine Position gef\"ahrlich nah am Tor? */
  bool isDangerousPos( const Vector& aPos) const;

  /** Ist der Ball im Strafraum ? */
  bool isBallInPenaltyArea(const Vector& aVector) const;
  
  /** Testet, ob durch Rueckwaertslaufen der Ball im Sichtfeld bleibt.
    Die Methode geht davon aus, da\3 target() das Laufziel enth\"alt. */
  bool isBackwardsRunUseful() const;
  
  /** Testet, ob sich ein TeamMate in dem Korridor befindet, der von der Linie 
    zwischen errechnetem target() und dem Ball geht. Die Breite Des Korridors
    entspricht der doppelten Distanz zwischen Torwart und seinem target(). */
  bool isTeamMateInBallZone() const;
  
  /** Der Grundnutzen des Wunsches. Dieser Teil der Nutzenberechnung ist nur 
    vom Weltmodell abhaengig.
    @return Ein Wert zwischen MAX_UTILITY und MIN_UTILITY. */
  int baseUtility() const;

  /** Liefert die Kosten, die durch die Einhaltung einer Nebenbedingung   
      entstehen. 
      @param constraint Zeiger auf die einzuhaltende Nebenbedingung
      @return Ein Wert zwischen NEUTRAL_COST und MAX_COST. */
  int constraintCost( const Constraint *constraint ) const;

  /** ForceContinuation wird neu definiert, weil das Umschalten des
      Sichtwinkels erzwungen werden muss. */
  bool forceContinuation() const;

  /** Beantwortet die Frage, ob noch ein eigener Spieler mit grosser
      Wahrscheinlichkeit zwischen Tor und Ball kommt. */
  bool ourPlayerDefense(const Situation& situation, ModelledBall& ball,
			       int nearestPlayerSteps) const;

  /** Generiert die Aktionen und setzt die Flags um zur HomePos zu kommen. */
  void computeHomePosition() const;

  /** Setzt alle zur Planberechnung notwendigen Flags auf Defaultparameter. */
  void setAllFlags() const;
 
  /** Testet, ob ein Catch moeglich ist */
  bool GoalieOption::isCatchPossible() const;
  /** Generiert den GoalieIntercept */
  int GoalieOption::computeGoalieIntercept() const;
  
  /** Berechnet das Ziel, wenn der Ball in der Penalty-Area ist. */
  void computePositionPlay() const;
  
  /** Berechnet die Zielposition fuer das Positionsspiel 
    @param addition Entfernung, die der Torwart dem Ball zusaetzlich entgegen laufen
    soll */
  void positionPlay(double addition = 0) const;
  
  /** Erzeugt einen Plan, der den Agenten auf die gew\"unschte Position bringt.
    Wenn m\"oglich, wird gebeamt, sonst mu\3 wird gotoRegion gemacht. Dabei
    werden Hindernisse immer umgangen. Ist der Agent schon an der gew\"unschten
    Position, dreht er sich zum Ball.
    @param target Absoluter Zielvektor
    @param targetRadius Zielradius f\"ur gotoRegion. Kann gebeamt werden, wird
    genau auf die gew\"unschte Position gebeamt.
    @param staminaLimit fuer gotoRegion. 
    @see gotoRegion
    @see beamToPosition */
  Plan movementTo( 
      const Vector& target, 
      const double targetRadius = TRIANGULATION_ACCURACY, 
      const double staminaLimit = 0,
      const int specialBehaviors = Skills::AVOID_OBSTACLES ) const;
};



inline void GoalieOption::setGoToGoalieHomePosition( bool on) const
{
  theGoToGoalieHomePosition = on;
}

inline bool GoalieOption::isGoToGoalieHomePosition() const
{
  return theGoToGoalieHomePosition;
}



inline void GoalieOption::setCatchAtOnce(bool on) const
{
  theCatchAtOnce = on;
}

inline bool GoalieOption::isCatchAtOnce() const
{
    return theCatchAtOnce;
}

inline void GoalieOption::setNormalView(bool on) const
{
    theNormalView = on;
}

inline bool GoalieOption::isNormalView() const
{
    return theNormalView;
}


inline void GoalieOption::setPositionPlay(bool on) const
{
    thePositionPlay = on;
}

inline bool GoalieOption::isPositionPlay() const
{
    return thePositionPlay;
}

inline void GoalieOption::setGoalieIntercept(bool on) const
{
  theGoalieIntercept = on;
}

inline bool GoalieOption::isGoalieIntercept() const
{
  return theGoalieIntercept;
}

inline void GoalieOption::setFreeKickSimStep( int newSimStep ) const
{
  theFreeKickSimStep = newSimStep;
}


inline int GoalieOption::freeKickSimStep() const
{
  return theFreeKickSimStep;
}


inline void GoalieOption::setAllFlags() const
{
  setGoalieIntercept( NO );
  setGoToGoalieHomePosition( NO );
  setCatchAtOnce( NO );
  setPositionPlay( NO );
  setNormalView( NO );
}

#endif
