#ifndef _MODELLEDOBJECT_H_
#define _MODELLEDOBJECT_H_

#include "MovingObject.h"
#include "Reliability.h"
#include "Common.h"
#include "Exception.h"
#include <math.h>

/**
  Eigene Annahmen ueber sich bewegendes Objekt in der Aussenwelt.
  ModelledObject ist eine abstrakte Superklasse. Moegliche Objekttypen sind
  Spieler und Ball.

  Ein ModelledObject ist die interne Repraesentation eines Objektes der
  Aussenwelt im Weltmodell. Diese Repraesentation ist immer in ABSOLUTEN
  Koordinaten, d.h. pos() gibt die Position des Objektes relativ zur
  Spielfeldmitte an. Die positive x-Achse zeigt in Richtung gegnerisches Tor,
  die y-Achse ist um 90 Grad (im mathematischen Sinn, also gegen den
  Uhrzeigersinn) gedreht. Ebenso ist speed() absolut zu interpretieren. 
  Der Wert
  gibt an, um wieviel sich die Position innerhalb eines Taktes ver\"andert.
  
  Zusaetzlich zu den Eigenschaften eines MovingObject
  hat ein ModelledObject Simulationsmoeglichkeiten, um seine eigene Position
  und
  Geschwindigkeit in der Zukunft zu bestimmen. Dazu besitzt es einen
  Verlangsamungsfaktor und Verlaesslichkeitsinformationen.

  {\small {\bf Quelltext}

  [\URL[Header]{src/ModelledObject.h} | [\URL[Implementierung]{src/ModelledObject.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  *************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: ModelledObject.h,v $
  *   $Revision: 1.21 $
  *   $Date: 1998/08/14 13:20:53 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class ModelledObject : public MovingObject
{
public:
  
  /** @name Interne Fehlerklassen. */
  //@{
    /** Allgemeine Fehlerklasse. Wird von [FEHLT] ausgel\"ost. */
    class ModelledObjectException: public Exception {
    public:
      ModelledObjectException(const String& aDescription )
        : Exception( aDescription ) {}
    };
  //@}

  /** @name Werte abfragen */
  //@{
    /** Liefert Verlaesslichkeit der Position. */
    const Reliability& posReliability() const;

    /** Liefert Verlaesslichkeit der Geschwindigkeit. */
    const Reliability& speedReliability() const;
  //@}
    
    
  /** @name Werte setzen */
  //@{
    /** Setzt Verlaesslichkeit der Position. */
    void setPosReliability( const Reliability& newPosReliability );

    /** Setzt Verlaesslichkeit der Geschwindigkeit. */
    void setSpeedReliability( const Reliability& newSpeedReliability );
      
    /** Reduziert die Verlaesslichkeit. */
    void reduceReliability();
  //@}
    
private:	
  
  /** Die Verlaesslichkeit der Position. Sie haengt von der anfaenglichen
    Verlaesslichkeit und von der Zahl der durchgefuehrten Simulationsschritte
    ab. */
  Reliability thePosReliability;
  
  /** Die Verlaesslichkeit der Geschwindigkeit. Sie haengt von der
    anfaenglichen Verlaesslichkeit und von der Zahl der durchgefuehrten
    Simulationsschritte ab. Dies ist auch gleichzeitig
    die Verl\"asslichkeit f\"ur die Blickrichtung. */
  Reliability theSpeedReliability;
  
protected:
  /** Protected Default-Konstruktor.
    Die Klasse ist abstrakt, aber der Konstruktor kann von den Nachkommen 
    genutzt werden. */
  ModelledObject();

  /** Neues ModelledObject mit Id, Position, Geschwindigkeit,
  Positions- und Geschwindikgeitsverl\"asslichkeit.
    @param id Die Objekt-Id. M\"ogliche Werte sind in den \Ref{Objekt-Ids}
      aufgelistet.
    @param newPosition Die Position.
    @param newSpeed Die Geschwindigkeit. 
    @param aPosReliability Die Verlaesslichkeit der Position. Als Default wird 
    RANDOM angenommen.
    @param aPosReliability Die Verlaesslichkeit der Geschwindikeit. Als Default
    wird RANDOM angenommen. */
  ModelledObject( int id,
    const Vector& aPos, 
    const Vector& aSpeed = Vector(),
		const Reliability& aPosReliability = Reliability(Reliability::RANDOM),
		const Reliability& aSpeedReliability = Reliability(Reliability::RANDOM) );
};

inline const Reliability& ModelledObject::posReliability() const 
{
  return thePosReliability;
}

inline const Reliability& ModelledObject::speedReliability() const 
{
  return theSpeedReliability;
}

inline void ModelledObject::setPosReliability(
    const Reliability& newPosReliability) 
{
  thePosReliability=newPosReliability;
}

inline void ModelledObject::setSpeedReliability(
    const Reliability& newSpeedReliability) 
{
  theSpeedReliability=newSpeedReliability;
}
 
inline void ModelledObject::reduceReliability() 
{
  thePosReliability.decrease();
  theSpeedReliability.decrease();
}

#endif






