#ifndef _RELIABILITY_H_
#define _RELIABILITY_H_

#include "Common.h"
#include "StorableObject.h"

/**
  Klasse, die die Verlaesslichkeit von Informationen erfasst.
  Damit Mehrfachvererbung vermieden wird, werden Klassen mit unsicheren 
  Informationen generell aus Klassen mit sicheren Informationen erzeugt, in dem
  eine Unterklasse angelegt wird, die zusaetzlich noch eine Instanzvariable vom
  Typ Reliability enthaelt.

  Die Verlaesslichkeit kann ganzahlige positive Werte annehmen. 0 entspricht
  dabei der hoechsten Verlaesslichkeit. Mit steigendem Wert sinkt die
  Verlaesslichkeit.

  Fuer einige Werte sind Konstanten definiert: (PERFECT,GOOD,BAD,RANDOM).
  RANDOM ist die maximale Unsicherheit, d.h. jeder Wert aus dem 
  Wertebereich ist genauso wahrscheinlich wie der tatsaechlich eingetragene.
  Jeder undefinierte Wert hat also die Reliability Reliability::RANDOM.

  {\small {\bf Quelltext}

  [\URL[Header]{src/Reliability.h} | [\URL[Implementierung]{src/Reliability.cc}]

  {\bf CVS Informationen zum Interface}

  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Reliability.h,v $
  *   $Revision: 1.18 $
  *   $Date: 1998/08/14 13:21:10 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class Reliability : public StorableObject
{
public:	
  
  /// perfekte Verl\"a\3lichkeit (gerade gesehen)
  const unsigned long PERFECT = 0;
  /// sehr gute Verl\"a\3lichkeit
  const unsigned long VERY_GOOD = 3;
  /// gute Verl\"a\3lichkeit
  const unsigned long GOOD = 6;
  /// schlechte Verl\"a\3lichkeit
  const unsigned long BAD = 9;
  /// zuf\"allig
  const unsigned long RANDOM = UNDEF_LONG/2;

  
  /** Konstruktor fuer ein neues Reliability-Objekt
    @param newValue Wert der Zuverlaessigkeit der im neuen Objekt gesetzt wird
    je hoeher der Wert, desto weniger Verlaesslich ist die Information */
  Reliability(unsigned long newValue); 
 
  /** liefert den aktuellen Verlaesslichkeitswert
    @return ein ganzahliger Wert zwischen 0 und UNDEF_LONG
    ( = LONG_MAX; definiert in Common.h )
    je hoeher der Wert, desto weniger Verlaesslich ist die Information */
  unsigned long get() const;

  /** setzt die Verlaesslichkeit auf einen neuen Wert
    @param newValue neuer Wert fuer die Verlaesslichkeit.
    je hoeher der Wert, desto weniger Verlaesslich ist die Information */
  void set(unsigned long newValue);
  
  /** erhoeht die Verlaesslichkeit der Information.
    Der Verlaesslichkeitswert wird hierbei um 1 vermindert. Er wird jedoch
    nicht kleiner als 0 ( maximale Verlaesslichkeit )       */
  void increase();

  /** vermindert die Verlaesslichkeit der Information.
    Der Verlaesslichkeitswert wird um 1 erhoeht. Er wird jedoch nicht groesser
    als UNDEF_LONG ( = LONG_MAX; definiert in Common.h ) */
  void decrease();

  /** Beschreibung als String fuer die Ausgabe auf einem Terminal. */
  virtual String toString() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
    darauf zurueck. */
  virtual StorableObject* copy() const;

private:
  unsigned long value;
  
};

inline unsigned long Reliability::get() const
{
  return value;
}

inline void Reliability::set(unsigned long newValue)
{
  value = newValue;
}

inline void Reliability::increase()
{
  if( value > 0 ) {
    value--;
  }
}

inline void Reliability::decrease()
{
  if( value < UNDEF_LONG ) {
    value++;
  }
}

#endif





