#ifndef _VECTOR_H_
#define _VECTOR_H_

#include <stream.h>
#include "Log.h"
#include "String.h"
#include "Exception.h"
#include "Common.h"
#include "StorableObject.h"

/*  Koordinatensysteme, mit denen ein Vektor initialisiert werden
  kann. Moegliche Werte sind {CARTESIAN, POLAR}. */
enum CoordinateSystemEnum {CARTESIAN, POLAR};

/*  Winkelmesssysteme, mit denen ein Winkel gesetzt werden
  kann. Moegliche Werte sind {RADIAN, DEGREE}. */
enum AngleSystemEnum {RADIAN, DEGREE};

/**
  Diese Klasse beschreibt einen 2D-Vektor. 
  Der Vektor akzeptiert Lesen und Schreiben seiner Werte sowohl in
  Polarkoordinaten als auch in kartesischen Koordinaten.

  Per Konvention werden die Werte so interpretiert:

  \begin{description}
  \item[Kartesisches Koordinatensystem:]
  x-Achse nach rechts, y-Achse nach oben. Bei Angabe als Paar: (x, y)

  \item[Polarkoordinatensystem]
  Winkel gegen den Uhrzeigersinn im Bogenmass [-Pi,Pi]. Bei Laenge 0 ist auch
  Winkel 0. Bei Angabe als Paar: (Laenge, Winkel)
  \end{description


  Intern werden momentan nur die kartesischen Koordinaten gefuehrt, die
  Polarkoordinaten jeweils dynamisch ausgerechnet.

  Die Klasse prueft vor Berechnungen <b>nicht</b>, ob die Komponenten gleich 
  UNDEF_DOUBLE sind. Werden zwei undefinierte Vektoren z.B. addiert, so ist das
  Ergebnis nicht unbedingt ein undefinierter Vektor (d.h. er antwortet auf
  isValid() nicht unbedingt mit NO).

  {\small {\bf Quelltext}

  [\URL[Header]{src/Vector.h} | [\URL[Implementierung]{src/Vector.cc}]

  {\bf CVS Informationen zum Interface}
  \begin{verbatim}
  **************************************************
  *
  * Aktueller Stand
  *
  *   $RCSfile: Vector.h,v $
  *   $Revision: 1.27 $
  *   $Date: 1998/08/14 13:21:29 $
  *   $Author: myritz $ 
  *
  **************************************************
  \end{verbatim} }*/
class Vector : public StorableObject
{
public:
  
  /** Interne Fehlerklasse. */
  class VectorException: public Exception {
  public:
    VectorException(const String& aDescription )
      : Exception( aDescription ) {}
  };
  
  /** Neuer Vektor.
    @param value1 Die x-Koordinate bei kartesischen, die Laenge bei
    Polarkoordinaten.
    @param value2 Die y-Koordinate bei kartesischen, der Winkel im Bogenmass 
    bei Polarkoordinaten.
    @param aCoordinateSystem. Das Koordinatensystem, in dem die beiden vorigen
    Werte zu interpretieren sind. CARTESIAN ist default.
    @param anAngleSystem. Das Winkelmesssystem, in dem der Winkel bei
    einer Polarkoordinatenangabe angegeben ist. RADIAN ist default.
    @param ist einer der Werte undefiniert wird der Vektor auf (UNDEF_DOUBLE,UNDEF_DOUBLE) gesetzt
    @exception VectorException falls: unbekanntes Koordinatensystem, unbekannte  Winkelart. */
  Vector( 
	 double value1, 
	 double value2, 
	 CoordinateSystemEnum aCoordinateSystem = CARTESIAN,
	 AngleSystemEnum anAngleSystem = RADIAN );
  
  /** Neuer undefinierter Vektor. */
  Vector();

  /** Gibt die x-Koordinate zurueck. */
  double x() const;

  /** Gibt die y-Koordinate zurueck. */
  double y() const;
  
  /** Setzt die x- und die y-Koordinate. */
  void setCartesian(const double anXValue, const double anYValue);
  
  /** Gibt die Laenge zurueck.(falls der Vektor ungueltig ist UNDEF_DOUBLE) */
  double length() const;
  
  /** Gibt den Winkel zurueck.(falls der Vektor ungueltig ist UNDEF_DOUBLE)
    @exception VectorException falls: unbekannte  Winkelart. */
  double angle( AngleSystemEnum anAngleSystem = RADIAN ) const;
  
  /** Setzt die Laenge und den Winkel des Vektors.
      Falls Laenge oder Winkel undefiniert ist, wird der Vektor auf (UNDEF_DOUBLE,UNDEF_DOUBLE) gesetzt
    @exception VectorException falls: unbekannte  Winkelart. */
  void setPolar( double aLength, double anAngle,
		 AngleSystemEnum anAngleSystem = RADIAN );

  /** Setzt Werte neu.
    @param value1 Die x-Koordinate bei kartesischen, die Laenge bei
    Polarkoordinaten.
    @param value2 Die y-Koordinate bei kartesischen, der Winkel bei
    Polarkoordinaten.
    @param aCoordinateSystem Das Koordinatensystem, in dem die beiden vorigen
    Werte zu interpretieren sind. CARTESIAN ist default.
    @param anAngleSystem. Das Winkelmesssystem, in dem der Winkel bei
    einer Polarkoordinatenangabe angegeben ist. RADIAN ist default.
    @exception VectorException falls: unbekanntes Koordinatensystem, unbekannte  Winkelart. */
  void set( 		
	   double value1, 
	   double value2, 
	   CoordinateSystemEnum aCoordinateSystem = CARTESIAN,
	   AngleSystemEnum anAngleSystem = RADIAN  );


  /** Prueft, ob der Vektor komplett definiert ist.. 
      @return YES falls beide Werte(x,y bzw. Laenge,Winkel) definiert sind, sonst NO */
  bool isValid() const;

  /** Gibt zur\"uck ob der Vektor mit einer negativen Distanz initialisiert
    wurde. Diese Methode wird f\"ur die dist- und dir-Change Parameter
    gebraucht, da sonst Information verloren geht. */
  bool wasInitializedWithNegativeDist() const;
    
  /** Setzt die Laenge auf 1. Der Winkel bleibt gleich */
  void normalize();
    
  /** Vektoraddition.  */
  Vector operator + (const Vector& v2) const;
  
  /** Vektorsubtraktion. */
  Vector operator - (const Vector& v2) const;
  
  /** die skalare Multiplikation von links */
  friend Vector operator * (double aScalarValue, const Vector& aVector);
  
  /** Addition des Vektor v2 zum aktuellen Wert. */
  const Vector& operator += (const Vector& v2);

  /** Subtraktion des Vektors v2 vom aktuellen Wert. */
  const Vector& operator -= (const Vector& v2);
  
  /** Vergleich des Vectors v2 mit dem aktuellen Vektor.
    Es wird \Ref{isAlmostEqual} verwendet.
    @return YES, wenn die beiden Vektoren praktisch identisch sind, sonst NO */
  bool operator == ( const Vector v2 ) const;

  /** Vergleich des Vectors v2 mit dem aktuellen Vektor auf Ungleichheit.
    Es wird \Ref{isAlmostEqual} verwendet.
    @return NO, wenn die beiden Vektoren praktisch identisch sind, sonst YES */
  bool operator != ( const Vector v2 ) const;
  
  /** Beschreibung als String fuer die Ausgabe auf einem Terminal.
      Es werden kartesische UND Polarkoordinate ausgegeben.
      Die Ausgabe der Polarkoordinaten erfolgt in Grad.
      Ausserdem wird auf ungueltige Vektoren hingewiesen. */
  virtual String toString() const;

  /** Legt eine Kopie des Objekts auf dem Heap an und liefert einen Zeiger
      darauf zurueck. */
  virtual StorableObject* copy() const;

private:
  /** Die x-Komponente. */
  double theX;
  /** Die y-Komponente. */
  double theY;
  /** Ob die Dist mit einen negativen Wert initialisiert wurde. */
  bool theWasInitializedWithNegativeDist;
};


inline double Vector::x() const
{
  return theX;
}

inline double Vector::y() const
{
  return theY;
}

inline void Vector::setCartesian(const double anXValue, const double anYValue)
{
  theX=anXValue;
  theY=anYValue;
}

inline bool Vector::isValid() const
{
  return !(theX == UNDEF_DOUBLE || theY == UNDEF_DOUBLE);
}

inline bool Vector::wasInitializedWithNegativeDist() const
{
  return theWasInitializedWithNegativeDist;
}

inline Vector operator * (double aScalarValue, const Vector& aVector) 
{
  return Vector(aScalarValue * aVector.x(), aScalarValue * aVector.y() );
}

inline Vector Vector::operator+ (const Vector& v2) const
{
  return Vector(x()+v2.x(),y()+v2.y());
}

inline Vector Vector::operator- (const Vector& v2) const
{
  return Vector(x()-v2.x(),y()-v2.y());
}

inline const Vector& Vector::operator+= (const Vector& v2)
{
  theX += v2.x();
  theY += v2.y();
  return *this;
}

inline const Vector& Vector::operator-= (const Vector& v2)
{
  theX -= v2.x();
  theY -= v2.y();
  return *this;
}

inline bool Vector::operator == ( const Vector v2 ) const
{
  return isAlmostEqual( x(), v2.x() ) && 
      isAlmostEqual( y(), v2.y() ) && 
      wasInitializedWithNegativeDist() == 
      v2.wasInitializedWithNegativeDist();
}
    
inline bool Vector::operator != ( const Vector v2 ) const
{
  return !operator==( v2 );
}

#endif

